# ##############################################
#
# SCRIPT TO COMPILE FPGA AND ADD TIMESTAMP
#
# ##############################################

proc hex2bin hex {
    set t [list 0 0000 1 0001 2 0010 3 0011 4 0100 \
	    5 0101 6 0110 7 0111 8 1000 9 1001 \
	    a 1010 b 1011 c 1100 d 1101 e 1110 f 1111 \
	    A 1010 B 1011 C 1100 D 1101 E 1110 F 1111]
    regsub {^0[xX]} $hex {} hex
    return [string map -nocase $t $hex]
}

proc bin2hex bin {
    array set t {
	0000 0 0001 1 0010 2 0011 3 0100 4
	0101 5 0110 6 0111 7 1000 8 1001 9
	1010 a 1011 b 1100 c 1101 d 1110 e 1111 f
    }
    set diff [expr {4-[string length $bin]%4}]
    if {$diff != 4} {
        set bin [format %0${diff}d$bin 0]
    }
    regsub -all .... $bin {$t(&)} hex
    return [subst $hex]
}



proc createTimestamp {} {

  # get current date & time
  # format of DateTime will be YYYY-MM-DD HH:MM:SS
  set DateTime [clock format [clock seconds] -format "%Y-%m-%d %H:%M:%S"]
 
  
  
  # YEAR: 2015 -> 2063 (6 bits)
  set yearHex [format %x [string range $DateTime 2 3]]
  set yearBin6bit [hex2bin $yearHex]
  
  # pad or trim to 6 bits
  if {[string length $yearBin6bit] == 4} {
      set yearBin6bit 00$yearBin6bit
  } else {
      set yearBin6bit [string range $yearBin6bit 2 7]
  } 
  
  
  # MONTH 1 -> 12 / 1 -> B  (4 bits)
  if {[string index $DateTime 5] == 0} {
      set monthHex [format %x [string index $DateTime 6]]
  } else {
      set monthHex [format %x [string range $DateTime 5 6]]
  }
  set monthBin4bit [hex2bin $monthHex]
  
  
  # DAY 1 -> 31 / 1 -> 1f  (5 bits)
  if {[string index $DateTime 8] == 0} {
      set dayHex [format %x [string index $DateTime 9]]
  } else {
      set dayHex [format %x [string range $DateTime 8 9]]
  }
  set dayBin5bit [hex2bin $dayHex]
  
  # pad or trim to 5 bits
  if {[string length $dayBin5bit] == 4} {
      set dayBin5bit 0$dayBin5bit
  } else {
      set dayBin5bit [string range $dayBin5bit 3 7]
  } 
 
  
  #  HOUR  0 -> 23 / 0 -> 17 (5 bits)
  if {[string index $DateTime 11] == 0} {
      set hourHex [format %x [string index $DateTime 12]]
  } else {
      set hourHex [format %x [string range $DateTime 11 12]]
  }
  set hourBin5bit [hex2bin $hourHex]
  
  # pad or trim to 5 bits
  if {[string length $hourBin5bit] == 4} {
      set hourBin5bit 0$hourBin5bit
  } else {
      set hourBin5bit [string range $hourBin5bit 3 7]
  }
  
  
  # MINUTES 0 -> 59 / 0 -> 3B (6 bits)
  if {[string index $DateTime 14] == 0} {
      set minHex [format %x [string index $DateTime 15]]
  } else {
      set minHex [format %x [string range $DateTime 14 15]]
  }
  set minBin6bit [hex2bin $minHex]
  
  # pad or trim to 6 bits
  if {[string length $minBin6bit] == 4} {
      set minBin6bit 00$minBin6bit
  } else {
      set minBin6bit [string range $minBin6bit 2 7]
  }
  
  
  # SECONDS 0 -> 59 / 0 -> 3B (6 bits)
  if {[string index $DateTime 17] == 0} {
      set secHex [format %x [string index $DateTime 18]]
  } else {
      set secHex [format %x [string range $DateTime 17 18]]
  }
  set secBin6bit [hex2bin $secHex]
  
  # pad or trim to 6 bits
  if {[string length $secBin6bit] == 4} {
      set secBin6bit 00$secBin6bit
  } else {
      set secBin6bit [string range $secBin6bit 2 7]
  }
    
  
  # 32bit binary timestamp - ddddd.mmmm.yyyyyy.hhhhh.mmmmmm.ssssss
  return [bin2hex $dayBin5bit$monthBin4bit$yearBin6bit$hourBin5bit$minBin6bit$secBin6bit]
  

}



# set pointer to results directory
set outputDir ../results

# create results directory
file mkdir $outputDir

# read source files
read_vhdl ../src/baudgen.vhd
read_vhdl ../src/receive.vhd
read_vhdl ../src/transmit.vhd
read_vhdl ../src/host.vhd
read_vhdl ../src/uart.vhd

# read xdc
read_xdc ../src/uart.xdc


# synthesis
synth_design -top uart -part xc7z020clg484-1
write_checkpoint -force $outputDir/post_synth.dcp

# implementation
opt_design
place_design
write_checkpoint -force $outputDir/post_place.dcp
route_design


set timestamp_hex [createTimestamp]
set_property BITSTREAM.CONFIG.USR_ACCESS $timestamp_hex [current_design]

write_checkpoint -force $outputDir/post_route.dcp


# generate bitstream - file name has timestamp as suffix
write_bitstream -force $outputDir/uart_$timestamp_hex.bit




